"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProjectConfigService = void 0;
const constants_1 = require("../constants");
const path = require("path");
const _ = require("lodash");
const ts = require("typescript");
const yok_1 = require("../common/yok");
class ProjectConfigService {
    constructor($fs, $logger, $injector) {
        this.$fs = $fs;
        this.$logger = $logger;
        this.$injector = $injector;
    }
    requireFromString(src, filename) {
        const m = new module.constructor();
        m.paths = module.paths;
        m._compile(src, filename);
        return m.exports;
    }
    get projectHelper() {
        return this.$injector.resolve("projectHelper");
    }
    detectInfo(projectDir) {
        const configJSFilePath = path.join(projectDir || this.projectHelper.projectDir, constants_1.CONFIG_FILE_NAME_JS);
        const configTSFilePath = path.join(projectDir || this.projectHelper.projectDir, constants_1.CONFIG_FILE_NAME_TS);
        const hasTS = this.$fs.exists(configTSFilePath);
        const hasJS = this.$fs.exists(configJSFilePath);
        if (!hasTS && !hasJS) {
            throw new Error(`You do not appear to have a ${constants_1.CONFIG_FILE_NAME_DISPLAY} file. Please install NativeScript 7+ "npm i -g nativescript". You can also try running "ns migrate" after you have the latest installed. Exiting for now.`);
        }
        if (hasTS && hasJS) {
            this.$logger.warn(`You have both a ${constants_1.CONFIG_FILE_NAME_JS} and ${constants_1.CONFIG_FILE_NAME_TS} file. Defaulting to ${constants_1.CONFIG_FILE_NAME_TS}.`);
        }
        return {
            hasTS,
            hasJS,
            configJSFilePath,
            configTSFilePath,
        };
    }
    readConfig(projectDir) {
        const { hasTS, configJSFilePath, configTSFilePath } = this.detectInfo(projectDir);
        let config;
        if (hasTS) {
            const rawSource = this.$fs.readText(configTSFilePath);
            const transpiledSource = ts.transpileModule(rawSource, {
                compilerOptions: { module: ts.ModuleKind.CommonJS },
            });
            const result = this.requireFromString(transpiledSource.outputText, configTSFilePath);
            config = result["default"] ? result["default"] : result;
        }
        else {
            const rawSource = this.$fs.readText(configJSFilePath);
            config = this.requireFromString(rawSource, configJSFilePath);
        }
        return config;
    }
    getValue(key) {
        return _.get(this.readConfig(), key);
    }
    setAppId(projectId, projectDir) {
        const { hasTS, configJSFilePath, configTSFilePath } = this.detectInfo(projectDir);
        const configPath = hasTS ? configTSFilePath : configJSFilePath;
        const rawSource = this.$fs.readText(configPath);
        this.$fs.writeFile(configPath, rawSource.replace(`org.nativescript.app`, projectId));
    }
}
exports.ProjectConfigService = ProjectConfigService;
yok_1.injector.register("projectConfigService", ProjectConfigService);
